title IBM PC Communications I/O Routines 
;
; $Header: comport.asm,v 1.7 90/02/19 11:43:58 gnu Exp $
;
; Orginal code -- Curt Klinsing
;
; Changes and updates -- Copyright (c) 1987 Tim Pozar
; Anyone can use this code for anything, but it is copyright by Tim
; and you must leave his copyright in the code.
;
; ver: 0
; rev: 2
; 19.Apr.87
;   Fixed bug in set_tty() so that the prameters for the port speced
; are pushed, not the default port.
;   Installed reset_tty().
;
; 13.Mar.1987
; This code is in a very early stage and should not be let out.
; Several other extensive functions are planned as well as changes
; to the current code.
;
; 20.Feb.87
;   Changed segment declarations and function names (eg. _function)
; to fit Microsoft C 4.0 and linker requirements.
;
; 1986
;   Ported from Lattice to MSC conventions (stack).
;
; FUNCTIONS CHANGED/ADDED --
; 19.Apr.87
; reset_tty()
;   Function to put the port back into the state it was when it was 
; first found by set_tty().  If set_tty() was not called it will not 
; change the settings of the port.  
;
; 17.Mar.87
; set_tty(p,b,l,d)
;   Function to find current settings of the port and set up serial 
; port for 'baud' and 'lcbyte', and enable/disable DTR.  This will set up 
; the port number base addressed passed to it (eg. 3F8h) and all functions
; will use this port until the function is used again.
;
; 13.Mar.87
; get_msr()
;   Function to read (get) the byte located in the Modem Status 
; Register (BASE+6).  The table below describes the byte returned.
;   bit  description
;    0   Delta Clear to Send (DCTS)
;        Indicates that the !CTS input to the chip has changed state
;        since the last time it was read by the processor.
;    1   Delta Data Set Ready (DDSR)
;        Indicates that the !DRS input to the chip has changed since 
;        last time it was read by the processor.
;    2   Trailing Edge Ring Indicator (TERI)
;        Indicates that the !RI input to the chip has changed from
;        an on (logical 1) to an off (logical 0) condition.
;    3   Delta Rx Line Signal detect (DRLSD)
;        Indicates that the !RLSD input to the chip has changed state.
; NOTE: Whenever bit 0, 1, 2, or 3 is set to a logical 1, a modem status
;       interrupt is generated.
;
;    4   Clear to Send (CTS)
;        This bit is the complement of the clear to send (!CTS) input.
;        If bit 4 (LOOP) of the MCR is set to a logical 1, this is 
;        equivalent to RTS in the MCR.
;    5   Data Set Ready (DSR)
;        This bit is the complement of the data set ready (!DSR) input.
;        If bit 4 (LOOP) of the MCR is set to a logical 1, this is 
;        equivalent to DTR in the MCR.
;    6   Ring Indicator (RI)
;        This bit is the complement of the ring indicator (!RI) input.
;        If bit 4 (LOOP) of the MCR is set to a logical 1, this is 
;        equivalent to OUT 1 in the MCR.
;    7   Receive Line Signal Detect (RLSD).
;        This bit is the complement of the received line signal detect
;        (!RLSD) input. If bit 4 (LOOP) of the MCR is set to a logical 1,
;        this is equivalent to OUT 2 in the MCR.
;
; 15.Mar.87
; get_lsr()
;   Function to read (get) the byte located in the Line Status 
; Register (BASE+5).  The table below describes the byte returned.
;
;   Currently this driver is set up for COM1 (BASE=3f8h).
;
; 25.Jul.87  Modifications (C)Copyright 1987  Garry Paxinos
;   Modified to use a fossil driver if it is present
;
; 14.Jan.88  gmp - added ability to send a break signal if using the fossil
;                  driver
;
; 15.Feb.90  Modification (C)Copyright 1990 Tim Pozar
;   Added code to send a break with comport in the outp_break() function.
;   Break should now work with both with and without FOSSILs (see above).
;

_TEXT   SEGMENT BYTE PUBLIC 'CODE'
_TEXT   ENDS

_DATA   SEGMENT BYTE PUBLIC 'DATA'
_fossil db      ?
_port   dw      ?
; baud rate curr,  300, 1200, 2400, 4800, 9600
FBaud	dw     0, 043h, 083h, 0A3h, 0C3h, 0E3h
CBaud   dw     0,  384,   96,   48,   24,   12 
_DATA   ENDS

CONST   SEGMENT BYTE PUBLIC 'CONST'
CONST   ENDS

_BBS    SEGMENT BYTE PUBLIC 'BBS'
_BBS    ENDS

DGROUP  GROUP   CONST, _BBS, _DATA
      ASSUME    CS: _TEXT, DS: DGROUP, SS: DGROUP, ES: DGROUP

_TEXT      SEGMENT
;
;A set of MSC 4.0 callable functions to support
;interrupt driven character I/O on the  IBM PC. Input
;is buffered, output is polled.
;
;added functions (TMP) --
public  _set_tty        ;push current settings, and initialize 
                        ;comm port to passed arguments.
public  _reset_tty      ;reset to settings that set_tty() found.
public  _get_msr        ;get MSR byte from port.
public  _get_lsr        ;get LSR byte from port.
;
;original functions --
public  _init_comm      ;initialize the comm port interupts,
public  _uninit_comm    ;remove initialization,
; NOTE: init_comm and uninit_comm and the interrupt handler have been
;       modified to handle the buffering of the transmission of 
;       charicters. (TMP)
public  _set_xoff       ;enable/disable XON/XOFF,
public  _get_xoff       ;read XON/XOFF state,
public  _rcvd_xoff      ;returns true if XOFF rcvd,
public  _sent_xoff      ;true if XOFF sent,
public  _inp_cnt        ;returns count of rcv chars,
public  _inp_char       ;get one char from buffer,
public  _inp_flush      ;flush input buffer,
public  _outp_char      ;output a character,
public  _outp_break     ;output a break signal
;
;
extrn	_sleep:near
;
;A better description can be found in the comment
;block  in each function.
;
;       assume  cs:pgroup
;
FALSE   EQU     0
TRUE    EQU     NOT FALSE
;
BASE    equ     03F8H   ;BASE FOR SERIAL BOARD (default)
;
EnblDRdy equ    01H     ; enable 'data-ready' interrupt bit
IntCtlr  EQU    21H     ;OCW 1 FOR 8259 CONTROLLER
EnblIRQ4 EQU    0EFH    ;Enable COMMUNICATIONS (IRQ4)
MaskIRQ4 EQU    10H     ;BIT TO DISABLE COMM INTERRUPT (IRQ4)
;
MDMCD   EQU     80H     ;mask for carrier dectect
SETBAU  EQU     80H     ;code for Divisor Latch Access Bit
MDMTBE  EQU     20H     ;8250 tbe flag
MDMBRK  EQU     40H     ;command code for 8250 break
LINMOD  EQU     03H     ;line mode=8 bit, no parity
MDMMOD  EQU     0BH     ;modem mode = DTR and RTS HIGH 
MDMDRL  EQU    0F4H     ;DTR and RTS low
STOP2   EQU     04H     ;BIT FOR TWO STOP BITS IF BAUD<300
RS8259  EQU     20H     ;OCW 3 FOR 8259
RSTINT  EQU     64H     ;SPECIFIC EOI FOR COMM INTERRUPT 
XOFF    EQU     13H     ;XOFF character
XON     EQU     11H     ;XON character
;
;       MISCELLANEOUS EQUATES
;
CR      EQU     13
LF      EQU     10
DosCall EQU     33      ;INTERRUPT NUMBER FOR DOS CALL  
CNSTAT  EQU     11      ;FUNCTION NUMBER FOR CONSOLE STATUS
CNIN    EQU     1       ;FUNCTION NUMBER FOR CONSOLE INPUT
BUFSIZ  EQU     512     ;Max NUMBER OF CHARS
SetIntVect  EQU 25H     ;SET INTERRUPT VECTOR FUNCTION NUMBER

;
; Communication parameters --
;
baud    equ     96      ; 1047 =  110 (are you kidding?)
                        ;  384 =  300
                        ;   96 = 1200
                        ;   48 = 2400
                        ;   24 = 4800
                        ;   12 = 9600
parity  equ     00000b  ;00000 = none
                        ;01000 = odd
                        ;11000 = even
stopbit equ     000b    ;  000 = 1 bit
                        ;  100 = 2 bits
wordlth equ     11b     ;   10 = 7 bits
                        ;   11 = 8 bits
lcbyte  equ     parity+stopbit+wordlth      ;line control byte
div_on  equ     80h     ;divisor latch access bit (DLAB)
;
;
DATAPORT dw     BASE    ;transmit/receive data port
LCR    	 dw     BASE+3  ; Line control register
IER      dw     BASE+1  ; Interrup Enable Register
MCR      dw     BASE+4  ;modem control register
MDMSTA   dw     BASE+5  ;line status register
MDMMSR   dw     BASE+6  ;modem status register
MDMBAD   dw     BASE    ;lsb baud resgister
LCR_BYTE db     lcbyte  ;pre "set_tty()" LCR byte
DLW_WORD dw     baud    ;pre "set_tty()" baud rate
PUSHPORT db     FALSE   ;flag to see if set_tty() has been run before
;
;       DUMP BUFFER, COUNT AND POINTER.  
;
CIRC_BUF DB     BUFSIZ DUP(?)   ;ALLOW 512 MaxIMUM BUFFERED CHARACTERS
BUF_TOP EQU     $ - 1           ;KEEP TRACK OF THE TOP OF THE BUFFER
CIRC_TOP DW     BUF_TOP         ;
;
CIRC_IN DW      OFFSET CIRC_BUF ;POINTER TO LAST CHAR. PLACED IN BUFFER
CIRC_CUR DW     OFFSET CIRC_BUF ;POINTER TO NEXT CHAR. TO BE RETRIEVED FROM
                                ; BUFFER
CIRC_CT DW      0               ;COUNT OF CHARACTERS USED IN BUFFER
SNT_XOFF DB     FALSE           ;FLAG TO CHECK IF AN XOFF HAS BEEN SEND
GOT_XOFF  DB    FALSE           ;FLAG TO CHECK IF AN XOFF HAS BEEN RECEIVED
SEE_XOFF  DB    FALSE           ;FLAT TO SEE IF WE ARE INTERESTED IN XON/XOFF


FOSSIL          equ     14h

COM1            equ     0
COM2            equ     1

SET_BAUD        equ     0
XMIT_CHAR       equ     1
READ_CHAR       equ     2
GET_STATUS      equ     3
INIT_DR         equ     4
DEINIT_DR       equ     5
SET_DTR         equ     6
GET_TICKS       equ     7
FLUSH_OUT       equ     8
PURGE_OUT       equ     9
PURGE_IN        equ     10
XMIT_NOWAIT     equ     11
READ_NOWAIT     equ     12
SET_FLOW        equ     15
WRITE_SCREEN    equ     19
WRITE_BIOS	equ	21
SEND_BREAK      equ     01Ah
BAUD_9600       equ     011100011b

;
; /* This function is used to set up the port number, baud rate, word length,
;  * and enable disable the modem.  If this function has not been run 
;  * previously, it will push the setings of the INS8250 chip.  These 
;  * settings can be 'popped' off by running the reset_tty() function.
;  * /
;
; set_tty(p,b,l,d)
; int p;        /* port number.        0 = whatever was last set or COM1 
;                                          if not specified */
; int b;        /* baud rate divisor.  0 = whatever its currently set for */
; int l;        /* lcr_byte            0 = whatever its currently set for */
; int d;        /* DTR and RTS         0 = default 
;                * Enable modem        1 = DTR, RTS and interrups on 
;                * Disable modem       2 = DTR, RTS and interrups off
;                *                         (this will drop the line)
;                */


_set_tty proc near
        push    bp
        mov     bp,sp
        sub     sp,8
        push    dx
        push    si

        mov     dx, COM1                ;figure out which comport
        mov     ax, word ptr [bp][4]    ; kinda kludgey but I didn't want
        cmp     ax, 03f8h               ; to mess with the original code
        je      st_comm1
        cmp     ax, 0
        je      st_sameport
        mov     dx, COM2
        jmp     st_comm1                ;misnomer but what the hey
st_sameport:
        mov     dx, _port
st_comm1:
        mov     _port, dx               ;check/init fossil
        mov     ah, INIT_DR
        int     FOSSIL
        cmp     ax, 01954h
        jne     st_nofossil             ;nofossil do normal stuff
        mov     _fossil, TRUE
        mov     ax, word ptr[bp][10]    ;set dtr accordingly
        cmp     ax, 0
        je      st_nodtr
        cmp     ax, 1
        je      st_raisedtr
        cmp     ax, 2
        je      st_lowerdtr
        mov     ah, FLUSH_OUT           ;flush output buffer
        mov     dx, _port
        int     FOSSIL
        jmp     st_nodtr

st_raisedtr:
        mov     ah, SET_DTR
        mov     al, 1
        mov     dx, _port
        int     FOSSIL
        jmp     st_nodtr
st_lowerdtr:
        mov     ah, SET_DTR
        mov     al, 0
        mov     dx, _port
        int     FOSSIL

st_nodtr:
        push    si              ; lookup baud rate in table
        mov     si, word ptr[bp][6]
        cmp     si, 0
        je      st_nobaud       ; i know, relative branch out of range
        shl     si, 1
        mov     ax, word ptr FBaud[si]
        mov     ah, SET_BAUD
        mov     dx, _port
        int     FOSSIL
st_nobaud:
        pop     si
        jmp     st_exit

st_nofossil:
        mov     _fossil, FALSE
        mov     ax,[bp+4]       ; get first argument (port base number)
        cmp     ax,0            ; let current setting ride?
        jz      skipport        ; yes
        mov     DATAPORT,ax     ; stuff 'em
        mov     MDMBAD,ax       ; +0
        inc     ax
        mov     IER,ax          ; +1
        inc     ax
        inc     ax
        mov     LCR,ax          ; +3
        inc     ax
        mov     MCR,ax          ; +4
        inc     ax
        mov     MDMSTA,ax       ; +5
        inc     ax
        mov     MDMMSR,ax       ; +6

        mov     al,PUSHPORT
        cmp     al,TRUE
        jz      skipport
        mov     dx,lcr
        in      al,dx
        or      al,div_on
        out     dx,al           ; set 8250 for baud rate read
        mov     dx,mdmbad
        in      al,dx
        inc     dx
        mov     ah,al
        in      al,dx
        mov     DLW_WORD,dx     ; save baud rate
        mov     dx,lcr
        in      al,dx           ; get line parameters
        and     al,7Fh          ; turn off divisor access bit 
        out     dx,al
        mov     LCR_BYTE,al     ; save them
        mov     al,TRUE
        mov     PUSHPORT,al
skipport:
        mov     ax,[bp+6]       ; get second argument (baud rate div)
        cmp     ax,0            ; let curret seting ride?
        jz      skipbs          ; yes
        push    si              ; lookup baud rate in table
        mov     si, ax
        shl     si, 1
        mov     ax, word ptr CBaud[si]
        pop     si
        push    ax              ; save it temp.
        mov     dx,LCR
        in      al,dx
        or      al,div_on
        out     dx,al           ; set 8250 for baud rate write
        mov     dx,MDMBAD
        pop     ax              ; get it back.
        out     dx,al           ; low byte divisor
        mov     al,ah
        inc     dx
        out     dx,al           ; high byte divisor
        mov     dx,LCR
        in      al,dx           ; get line parameters
        and     al,7Fh          ; turn off divisor access bit 
        out     dx,al
skipbs:
        mov     ax,[bp+8]       ; get third argument (lcr byte)
        cmp     ax,0            ; default?
        jz      skiplcb         ; yes, skip
        mov     dx,lcr
        out     dx,al           ; set line control reg.
skiplcb:
        mov     ax,[bp+10]      ; get forth argument (DTR and RTS)
        cmp     ax,0            ; default?
        jz      flsh            ; yes, skip it all
        cmp     ax,1            ; turn on?
        jnz     skipdron        ; no, next test
        mov     dx,mcr
        in      al,dx
        or      al,mdmmod
        out     dx,al           ; on ...
skipdron:
        cmp     ax,2            ; turn off?
        jnz     flsh            ; no, next test
        mov     dx,mcr
        in      al,dx
        and     al,mdmdrl
        out     dx,al           ; off ...
flsh:   mov     dx,DATAPORT     ; flush the port 
        in      al,dx
        mov     dx,mdmsta
        in      al,dx
        and     al,1
        jnz     flsh
        call    _init_comm

st_exit:      
        pop     si
        pop     dx
        mov     sp,bp
        pop     bp
        ret

_set_tty endp


_reset_tty proc near
        push    bp
        push    dx
        cmp     _fossil, FALSE
        je      rt_nofossil

        mov     ah, SET_DTR             ;lower dtr
        mov     al, 0
        mov     dx, _port
        int     FOSSIL
        mov     ah, DEINIT_DR           ;deinit fossil
        mov     dx, _port
        int     FOSSIL
        mov     _fossil, FALSE
        jmp     rt_exit

rt_nofossil:
        mov     dx,lcr
        in      al,dx
        or      al,div_on
        out     dx,al
        mov     dx,mdmbad
        mov     ax,DLW_WORD
        inc     dx
        out     dx,al
        dec     dx
        mov     al,ah
        out     dx,al
        mov     dx,lcr
        mov     al,LCR_BYTE
        and     al,7Fh
        out     dx,al
        call    _uninit_comm
        
rt_exit:
        pop     dx
        pop     bp
        ret

_reset_tty endp

_get_msr proc near
        push    bp
        push    dx
        cmp     _fossil, FALSE
        je      gm_nofossil

        mov     ah, GET_STATUS
        mov     dx, _port
        int     FOSSIL
        and     ax, 00ffh
        jmp     gm_exit

gm_nofossil:
        push    ds              ; save data segment
        push    cs
        pop     ds

        xor     ax,ax
        mov     dx,MDMMSR
        in      al,dx
        pop     ds

gm_exit:
        pop     dx
        pop     bp
        ret

_get_msr endp

_get_lsr proc near
        push    bp
        push    ds              ; save data segment
        push    cs
        pop     ds

        cmp     _fossil, FALSE
        je      gl_nofossil

        mov     ah, GET_STATUS
        mov     dx, _port
        int     FOSSIL
        xchg    ah, al
        and     ax, 00ffh
        jmp     gl_exit

gl_nofossil:
        xor     ax,ax
        mov     dx,MDMSTA
        in      al,dx

gl_exit:
        pop     ds
        pop     bp
        ret

_get_lsr endp

;
; set_xoff(flag)         Enable (flag != 0) or disable
;int flag;              (flag == 0) XON/ XOFF protocol
;                       for the character input stream.
;If enabled, an XOFF will be sent when  the buffer
;reaches 3/4 full. NOTE: an XON will not be sent auto-
;matically. Your program must do it when it sees
;the _rcvd_xoff() flag,  and ready for more chars.
;
_set_xoff proc near
        push    bp
        PUSH    DS              ;SAVE DATA SEGMENT
        mov     bx,[bp+6]       
        push    cs
        pop     ds              ; move code seg addr to data seg reg.
        cmp     bx,0
        jnz     to_on
        mov     see_xoff,FALSE
        jmp     done1
to_on:  mov     see_xoff,TRUE
done1:  pop     ds
        pop     bp
        ret
_set_xoff endp
;
;flag = get_xoff()      Returns the current setting
;                       of the XON/ XOFF flag set
;by set_xoff(), above.
;
_get_xoff proc near
        push    bp
        push    ds              ; save data reg
        push    cs
        pop     ds              ; move code seg addr to data seg reg.
        xor     ax,ax
        mov     al,see_xoff
        pop     ds
        pop     bp
        ret
_get_xoff endp
;
;flag = sent_xoff();    Returns true if an XOFF
;                       character was sent, indicating
;the receive buffer is  3/4 full.
;
_sent_xoff proc  near
        push    bp
        push    ds              ; save data reg
        push    cs
        pop     ds              ; move code seg addr to data seg reg.
        xor     ax,ax
        mov     al,snt_xoff
        pop     ds
        pop     bp
        ret
_sent_xoff endp
;
; rcvd_xoff()            Returns true if an XOFF was
;                       received; will return false as
;soon as an XON is received. Does not effect data output,
;only indicates the above. (Obviously useless for binary
;data.)
;
_rcvd_xoff proc  near
        push    bp
        push    ds              ; save data reg
        push    cs
        pop     ds              ; move code seg addr to data seg reg.
        xor     ax,ax
        mov     al,got_xoff
        pop     ds              ; restore data reg
        pop     bp
        ret
_rcvd_xoff endp
;
;count = inp_cnt()       Returns the number of characters
;                       available in the input buffer.
;

_inp_cnt proc near       
        push    bp
        cmp     _fossil, FALSE
        je      icn_nofossil

        mov     ah, GET_STATUS
        mov     dx, _port
        int     FOSSIL
        xchg    ah, al
        and     ax, 1           ; data available in buffer
        jmp     icn_exit

icn_nofossil:
        push    ds              ; save data segment
        push    cs
        pop     ds              ; move code seg addr to data seg reg
        mov     ax,circ_ct
        pop     ds
icn_exit:
        pop     bp
        ret
_inp_cnt endp
;
; inp_flush()    Flush the input buffer.
;
_inp_flush proc  near    
        push    bp
        push    dx
        cmp     _fossil, FALSE
        je      if_nofossil

        mov     ah, PURGE_IN
        mov     dx, _port
        int     FOSSIL
        jmp     if_exit

if_nofossil:
        push    ds              ; save data reg
        push    cs
        pop     ds              ; move code seg addr to data seg reg.
        mov     bx,offset circ_buf
        mov     circ_in,bx      
        mov     circ_cur,bx
        xor     ax,ax
        mov     circ_ct,ax
        pop     ds

if_exit:
        pop     dx
        pop     bp
        ret
_inp_flush endp

; --------- Init -----------------------------------
; Program initialization:
;   --  Set up vector for RS232 interrupt (0CH)
;   --  Enbl IRQ4
;   --  Enbl RS232 interrupt on data ready
;
; ---------------------------------------------------

_init_comm proc  near
        push    bp
        cli

;  ---- Set up  INT x'0C' for IRQ4

        push    ds
        push    cs
        pop     ds              ;cs to ds
        mov     dx,offset IntHdlr ;relative adddres of interrupt handler
        mov     al,0cH          ;interrupt number for comm.
        mov     ah,SetIntVect   ;function number for setting int vector
        int     DosCall         ;set interrupt in 8086 table
        pop     ds              ;restore DS

;  ---- Enbl IRQ4 on 8259 interrupt controller

        cli

        in      al,IntCtlr      ; get current masks 
        and     al,EnblIRQ4     ; Reset IRQ4 mask
        out     IntCtlr,al      ; And restore to IMR

;  ---   Enbl 8250 data ready interrupt

        mov     dx,LCR          ; DX ==> LCR
        in      al,dx           ; Reset DLAB for IER access
        and     al,7FH
        out     dx,al
        mov     dx,IER          ; Interrupt Enbl Register
        mov     al,EnblDRdy     ; Enable 'data-ready' interrupt
        out     dx,al

;  ---   Enbl OUT2 on 8250

        mov     dx,MCR          ; modem control register        
        in      al,dx           ; Enable OUT2
        or      al,08h            ; find out what is in there and
        out     dx,al            ; enable the DTR

        sti

        pop     bp
        ret
_init_comm endp
;
; uninit_comm()          Removes the interrupt structure
;                       installed by _init_comm(). Must be
;done before passing control to the DOS, else chars received
;will be stored into the next program loaded!
;
_uninit_comm proc near
        push    bp
; ---   Disable IRQ4 on 8259

        cli
        in      al,IntCtlr      ;GET OCW1 FROM 8259
        or      al,MaskIRQ4     ;DISABLE COMMUNICATIONS INTERRUPT
        out     IntCtlr,al

; ---   Disable 8250 data ready interrupt
        
        mov     dx,LCR          ; DX ==> LCR
        in      al,dx           ; Reset DLAB for IER access
        and     al,7FH
        out     dx,al
        mov     dx,IER          ; Interrupt Enbl Register
        mov     al,0            ; Disable all 8250 interrupts
        out     dx,al

;  ---   Disable OUT2 on 8250

        mov     dx,MCR          ; modem control register        
        mov     al,0            ; Disable OUT2
        out     dx,al

        sti
        pop     bp
        ret
_uninit_comm endp
;
;char  inp_char()      Return a character from the input
;                      buffer. Assumes you have called
;inp_cnt() to see if theres any characters to get.
;
_inp_char proc near      
        push    bp
        push    dx
        cmp     _fossil, FALSE
        je      ic_nofossil

        mov     ah, READ_CHAR
        mov     dx, _port
        int     FOSSIL
        and     ax, 00ffh
        jmp     ic_exit

ic_nofossil:
        push    ds              ; save data reg
        push    cs
        pop     ds              ; move code seg addr to data seg reg.
        mov     bx,circ_cur
        xor     ax,ax
        mov     al,[bx]         ;get next char from circ_buf
        DEC     circ_ct         ;decrement circ_buf COUNT
        CMP     bx,circ_top     ;ARE WE AT THE TOP OF THE circ_buf?
        JZ      reset_cur       ;JUMP IF SO
        INC     bx              ;ELSE, BUMP PTR
        JMP SHORT upd_cur
reset_cur:
        mov     bx,OFFSET circ_buf      ;RESET circ_in TO BOTTOM OF BUF.
upd_cur:
        mov     circ_cur,bx             ;SAVE NEW PTR
        xor     cx,cx
        mov     cl,see_xoff     ;check if interested in xon/xoff
        cmp     cl,TRUE
        jnz     clnup2          ;not interested, so goto return
        cmp     snt_xoff,TRUE   ;have we sent an xoff?
        jnz     clnup2          ;no, so return
        cmp     circ_ct,80h     ;yes, so see in buf is now emptying
        jg      clnup2          ;not empty enuf to send xon, jump to ret
        mov     snt_xoff,FALSE
        mov     cl,XON
        push    ax              ; save char
        call    comout
        pop     ax
clnup2: pop     ds              ;GET BACK ENTERING DS

ic_exit:
        pop     dx
        pop     bp
        ret
_inp_char endp
;
; outp_char(c)           Output the character to the
;char c;                serial port. This is not buffered
;                       or interrupt driven. It is now, if using a 
;                       fossil driver!
;
_outp_char proc  near
        push    bp
        mov     bp,sp
	push    dx
        cmp     _fossil, FALSE
        je      oc_nofossil

        mov     ah, XMIT_CHAR
        mov     dx, _port
        mov     al, byte ptr [bp][4]
        int     FOSSIL
        jmp     oc_exit

oc_nofossil:
        mov     cl,[bp+4]
        sti
        call    comout

oc_exit:
        pop     dx
        pop     bp
        ret
_outp_char endp


;
; send a break signal out port
;
_outp_break proc  near
        push    bp
        mov     bp,sp
	push    dx
        cmp     _fossil, FALSE
        je      ob_nofossil

        mov     ah, SEND_BREAK
        mov     dx, _port
        mov     al, 1
        int     FOSSIL
	
	mov     ax, 2
	push    ax
        call    _sleep
        pop     ax  
	
        mov     ah, SEND_BREAK
        mov     dx, _port
        mov     al, 0
        int     FOSSIL
	mov     ax, 2
	        
        jmp     ob_exit

ob_nofossil:
	xor	ax,ax			;zero ax
	mov	dx,LCR			;load LCR register
	in	al,dx
	or	al,40h			;set "Set Break" bit to one
	out	dx,al			;stuff it back
	
	mov     ax, 1			;wait a sec.
	push    ax
        call    _sleep
        pop     ax  

	xor	ax,ax			;zero ax
	mov	dx,LCR			;load LCR register
	in	al,dx
	and	al,0bfh			;set "Set Break" bit to zero 
	out	dx,al			;stuff it back

ob_exit:
        pop     dx
        pop     bp
        ret
_outp_break endp
;
;Local  subroutine: output CL to the port.
;
comout: mov     dx,MDMSTA       
        in      al,dx           ; get 8250 status
        and     al,MDMTBE       ; check for transmitter ready
        jz      comout          ; jump if not to wait
        mov     al,cl           ; get char to al
        mov     dx,DATAPORT     
        out     dx,al           ; output char to 8251
        ret
;
;       RECEIVE INTERRUPT HANDLER (CHANGED TO PLACE CHARACTERS IN A
;        CIRCULAR circ_buf AND TO SEND AN XOFF IF THE circ_buf IS MORE THAN
;        3/4 FULL - S.G.)
;
IntHdlr:
        CLI
        push    cx
        push    dx
        push    bx
        push    ax
        push    ds
        mov     ax,cs           ;get cur code segment
        mov     ds,ax           ;and set it as data segment
        mov     bx,circ_in      ;GET circ_buf IN PTR
        mov     DX,DATAPORT     ;GET DATA PORT NUMBER
        IN      AL,DX           ;GET RECEIVED CHARACTER
;       push    ax
;       push    dx   
;       xor     ax,ax
;       xor     dx,dx
;       mov     dl,al
;       mov     ah,2
;       int     DosCall
;       pop     dx
;       pop     ax
        xor     cx,cx
        mov     cl,see_xoff     ;check if interested in xon/xoff
        cmp     cl,TRUE
        jnz     ck_full         ;not interested goto ck if buf full
        mov     cl,al           ;put char in cl for testing
        and     cl,7fh          ;turn off any parity bits 
        cmp     cl,XOFF         ;see if we got an xoff
        jnz     ck_xon
        mov     got_Xoff,TRUE   ; code for handling xon/xoff from remote
        jmp     clnup
ck_xon: cmp     cl,XON
        jnz     reg_ch
        mov     got_Xoff,FALSE
        jmp     clnup
;
;Normal character; not  XON/XOFF, or XON/XOFF disabled.
;
reg_ch: test    snt_Xoff,TRUE   ;SEE IF sentXoff IS SET
        jnz     ck_full         ;IF SO, DON'T SEND ANOTHER XOFF
        CMP     circ_ct,(BUFSIZ * 3)/4  ;ALLOW BUF TO BECOME 3/4 FULL BEFORE
                                        ; SENDING XOFF
        jb      savch           ;IF IT'S OK, CONTINUE
        push    ax              ;SAVE CHARACTER
        mov     CL,XOFF         ;GET XOFF CHARACTER
        mov     snt_Xoff,TRUE  ;RESET sentXoff
        call    comout          ; AND SEND IT
        pop     ax              ;RETRIEVE CHARACTER
        JMP SHORT savch         ;IF WE'RE HERE, THE circ_buf HAS BUFSIZ-80H
                                ;  CHARACTERS
ck_full:
        CMP     circ_ct,BUFSIZ  ;SEE IF circ_buf ALREADY FULL
        JZ      clnup           ; JUMP IF SO, DO NOT PLACE CHARACTER IN BFR
savch:                          
        mov     [bx],AL         ;SAVE NEW CHARACTER IN circ_buf
        inc     circ_ct         ;BUMP circ_buf COUNT
        CMP     bx,circ_top     ;ARE WE AT THE TOP OF THE circ_buf?
        JZ      reset_in        ;JUMP IF SO
        inc     bx              ;ELSE, BUMP PTR
        JMP SHORT into_buf
reset_in:
        mov     bx,OFFSET circ_buf      ;RESET circ_in TO BOTTOM OF BUF.
into_buf:
        mov     circ_in,bx              ;SAVE NEW PTR
clnup:
;        mov     AL,RSTINT
        mov     al, 020h
        OUT     RS8259,AL       ;ISSUE SPECIFIC EOI FOR 8259
        pop     ds              ;GET BACK ENTERING DS
        pop     ax
        pop     bx
        pop     dx
        pop     cx
        sti
        iret

_TEXT   ENDS

end
