/*
 * Comport.h
 *
 * @(#) comport.h	Version hoptoad-1.5	89/04/26
 *
 * Copyright (C) Tim M. Pozar 1987
 * Anyone can use this code for anything, but it is copyright by Tim
 * and you must leave his copyright in the code.
 */

/*
 * set_tty(p,b,l,d)
 *   Function to set up the port number, baud rate, word length,
 * and enable/disable the modem.  If this function has not been run
 * previously, it will push the settings of the INS250 chip.  These
 * settings can be 'popped' off by running the reset_tty() funtion.
 */

/*
 * int p;
 */
#define COMM1 1016      /* 3F8h */
#define COMM2  760      /* 2F8h */

/*
 * int b;
 */
#define B300    1      /* 300 bps */
#define B1200   2      /* 1200 bps */
#define B2400   3      /* 2400 bps */
#define B4800   4      /* 4800 bps */
#define B9600   5      /* 9600 bps */

#define  B50     2304
#define  B75     1536
#define  B110    1047
#define  B134     860
#define  B150     768
#define  B200     576
#define  B600     192
#define  B1800     64
#define  B19200     6
#define  B38400     3
#define  EXTB       0  /* do external clocking */


/*
 * int l;
 *   These defines should be added together to create a byte.
 *   eg.:
 *     l = CS7+CSTOPB  will create a path with 7 bits,
 *         and 2 stop bits.
 *   Standard path would be:
 *     l = CS8 which would create a path with 8 bits, 1 stop
 *         bit, and no parity.
 */
#define CS7      2      /* 7 bit word */
#define CS8      3      /* 8 bit word */
#define CSTOPB   4      /* 2 stop bits, else one */
#define PARODD   8      /* odd parity, else none */
#define PAREVE  24      /* even parity, else none */

/*
 * int d;
 */
#define MENBL    1      /* Turn on DTR, RTS and interrups */
#define MDSBL    2      /* Turn off DTR, RTS and interrups */

/*
 * get_msr()
 *   Function to read (get) the byte located in the Modem Status 
 * Register (3FEh).  The table below describes the byte returned.
 *   bit  description
 *    0   Delta Clear to Send (DCTS)
 *        Indicates that the !CTS input to the chip has changed state
 *        since the last time it was read by the processor.
 *    1   Delta Data Set Ready (DDSR)
 *        Indicates that the !DRS input to the chip has changed since 
 *        last time it was read by the processor.
 *    2   Trailing Edge Ring Indicator (TERI)
 *        Indicates that the !RI input to the chip has changed from
 *        an on (logical 1) to an off (logical 0) condition.
 *    3   Delta Rx Line Signal detect (DRLSD)
 *        Indicates that the !RLSD input to the chip has changed state.
 * NOTE: Whenever bit 0, 1, 2, or 3 is set to a logical 1, a modem status
 *       interrupt is generated.
 *
 *    4   Clear to Send (CTS)
 *        This bit is the complement of the clear to send (!CTS) input.
 *        If bit 4 (LOOP) of the MCR is set to a logical 1, this is 
 *        equivalent to RTS in the MCR.
 *    5   Data Set Ready (DSR)
 *        This bit is the complement of the data set ready (!DSR) input.
 *        If bit 4 (LOOP) of the MCR is set to a logical 1, this is 
 *        equivalent to DTR in the MCR.
 *    6   Ring Indicator (RI)
 *        This bit is the complement of the ring indicator (!RI) input.
 *        If bit 4 (LOOP) of the MCR is set to a logical 1, this is 
 *        equivalent to OUT 1 in the MCR.
 *    7   Receive Line Signal Detect (RLSD) or Carrier Detect (CD).
 *        This bit is the complement of the received line signal detect
 *        (!RLSD) input. If bit 4 (LOOP) of the MCR is set to a logical 1,
 *        this is equivalent to OUT 2 in the MCR.
 */
#define  DCTS       1
#define  DDSR       2
#define  TERI       4
#define  DRLSD      8
#define  CTS       16
#define  DST       32
#define  RI        64
#define  RLSD     128   /* Also know as ... */
#define  CD       128   

/*
 * get_lsr()
 *   Function to read (get) the byte located in the Line Status 
 * Register (3FDh).  The table below describes the byte returned.
 *   bit  description
 *    0   Data Ready (DR)
 *        DR is set to a logical 1 whenever a complete incoming character 
 *        has been received and transferred into the receiver buffer 
 *        register.  Bit 0 may be reset to a logical 0 either by the 
 *        processor reading the data in the receiver buffer register or by 
 *        writing a logical 0 into it from the processor. 
 *    1   Overrun Error (OE)
 *        Indicates that data in the receiver buffer register was not read
 *        by the processor before the next character was transferred into 
 *        the receiver buffer register, thereby destroying the previous 
 *        character.  The OE indicator is reset whenever the processor 
 *        reads the contents of the line status register.
 *    2   Parity error (PE)
 *        Indicates that the received data character does not have the 
 *        correct even or odd parity, as selected by the even parity-select 
 *        bit.  The PE bit is set to a logical 1 upon detection of a parity 
 *        error and is reset to a logical 0 whenever the processor reads the 
 *        contents of the line status register.
 *    3   Framing error (FE)
 *        Indicates that the received character did not have a valid stop 
 *        bit.  Bit 3 is set to a logical 1 whenever the stop bit following
 *        the last data bit or parity is detected as a zero bit (spacing 
 *        level0.
 *    4   Break interrupt (BI)
 *        Bit 4 is set to a logical 1 whenever the received data input is 
 *        held in the spacing (logical 0) state for longer that a full word
 *        transmission time (that is, the total time of start bit+data bits+ 
 *        stop bits.)
 * Note: Bits 1 through 4 are the error conditions that produce a receiver 
 *       line status interrupt whenever any of the corresponding 
 *       conditions are detected.
 *
 *    5   Transmitter Holding Register Empty (THRE)
 *        Indicates that the INS8250 is ready to accept a new character 
 *        for transmission.  In addtion, this bit causes the INS8250 to 
 *        issue an interrupt to the processor when the transmit holding 
 *        register empty interrupt enable is set high.  The THRE bit is 
 *        set to a logical 1 when a character is transferred from the 
 *        transmitter holding register into the transmitter shift register.  
 *        The bit is reset to logical 0 concurrently with the loading of 
 *        the transmitter holding register by the processor.
 *    6   Transmitter Shift Register Empty (TSRE)
 *        Bit 6 is set to a logical 1 whenever the transmitter shift
 *        register is idle.  It is reset to a logical 0 upon a data 
 *        transfer from the transmitter holding register to the transmitter
 *        shift register.  Bit 6 is a read-only bit.
 *    7   This bit is permanently set to a logical 0.
 */
#define  DR         1
#define  OE         2
#define  PE         4
#define  FE         8
#define  BI        16
#define  THRE      32
#define  TSRE      64
