/*
    $Header: includes.h,v 1.17 90/02/22 19:36:29 gnu Exp $
    Copyright 1989 Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Include files for various supported systems.
 *
 * This file exists so that every other file in gnuucp doesn't have
 * to be changed to the random local includes on random micros.
 *
 * Note that SYSTEM-DEPENDENT include files should not, in general,
 * be included here.  The idea here is to provide a generic environment
 * where things like stdio, malloc, strchr, etc work.  System-dependent
 * header files (e.g. for serial port control) should only be included
 * in the "sysdep.XXX" file for your system.
 *
 * Note that NAMESIZE should be the max length of a file name, including
 * all its directories, drive specifiers, extensions, and the like.
 * E.g. on a Unix with 14-char file names, NAMESIZE is several hundred
 * characters, since the 14-char names can be nested.
 */
#ifdef BSD
/* Unix Berserkeley systems */
#include <stdio.h>
#include <ctype.h>
#include <strings.h>
#include <sys/param.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <sys/dir.h>
#include <sys/stat.h>

extern char *malloc(), *calloc(), *realloc();
#define	strchr	index
#define strrchr	rindex
extern char *strtok();
extern long atol();
extern long time();
extern void perror(), exit();
extern void qsort();

#define	UNIX
#define	NAMESIZE	MAXPATHLEN
#endif

#ifdef SYSV
/* Unix System V */
#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <time.h>

extern char *malloc(), *calloc(), *realloc();
extern long atol();
extern long time();
extern void perror(), exit();
extern void qsort();

#define	UNIX
#define	NAMESIZE	512
#endif

#ifdef UNIX
/* Stuff common to all Unix systems */
#define	remove	unlink	/* Unix-ism for removing a file */
#define	BUFFEREDIO
#define	MULTITASK
#define	STDIN		0
#define	O_BINARY	0	/* No distinction between text and binary */
#endif

#ifdef CPM
/* CP/M-80 */
#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>

#define	NAMESIZE	50		/* No directories... */
#endif

#ifdef MSDOS
/* Microsoft DOS */
#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <process.h>
#include <string.h>

#ifdef MSC
#include <direct.h>
#include <memory.h>
#include <sys/types.h>
#include <sys/stat.h>
#endif

#ifdef TURBOC
#include <dir.h>
#include <alloc.h>
#endif

/* Turn on support for the interrupt driven comm port routines */
#define	COMPORT

#ifdef COMPORT
#define	BUFFEREDIO
#include "comport.h"
void handler();
#endif

extern long atol();
extern long time();
extern void perror(), exit();
extern void qsort();

#define	NAMESIZE	128		/* full path size */
#endif

#ifdef ST
/* Atari ST */
#include <stdio.h>
#include <ctype.h>
#include <osbind.h>
#include <signal.h>
#include <setjmp.h>
#include <time.h>

extern char *malloc(), *calloc(), *realloc();
#define	strchr	index
#define strrchr	rindex
extern char *strchr(), *strrchr(), *strtok();
extern long atol();
extern long time();
extern void perror(), exit();
extern void qsort();

#define NAMESIZE	128	/* full path size */

#define	O_WRONLY	1	/* Open file for writing */
#define	O_APPEND	0	/* No append only mode */

#endif

#ifdef VMS

#include <ctype.h>
#include <dcdef.h>
#include <descrip.h>
#include <dvidef.h>
#include <errno.h>
#include <file.h>
#include <file>
#include <iodef.h>
#include <math.h>
#include <rmsdef.h>
#include <setjmp.h>
#include <signal.h>
#include <ssdef.h>
#include <stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <tt2def.h>
#include <ttdef.h>
#include <string.h>

extern long atol();

#define NAMESIZE 255
#define	LOGCLOSE	/* Logfile must be closed; VMS locks it when open */

#define	time_t	unsigned
#define remove delete	/* Remove a file */
#define qsort pqsort	/* Our own version */

#endif

#ifdef MAC
/* Macintosh under Lightspeed C */
#include <MacTypes.h>
#include <OsUtil.h>
#include <stdio.h>
#include <ctype.h>
#include <setjmp.h>
#include <io.h>

extern char *strtok();
extern char *malloc(), *calloc(), *realloc();
extern char *strchr(), *strrchr();
extern long atol();

#define	NAMESIZE	512		/* Picked at random */

/*
 * What to do immediately upon entry to gnuuio
 */
#define	GNUUIO_MAIN_HOOK \
	enter_gnuuio(&argc, &argv)

#endif



#ifdef MAC_MPW
/* Macintosh Programmers Workshop. */
#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include <setjmp.h>
#include <uumac_mpw.h>

extern char *malloc(), *calloc(), *realloc();
extern long atol();

#define NAMESIZE 512 /* Hat. */

#define	abort()	exit(1);
#endif

#ifdef AMIGA
#  include <exec/types.h>
#  include <fcntl.h>
#  include <exec/exec.h>
#  include <devices/serial.h>
#  include <devices/keymap.h>
#  include <devices/timer.h>
#  include <libraries/dos.h>
#  include <signal.h>
#  include <stdio.h>
#  include <ctype.h>
#  include <time.h>

extern long atol();

#  define NAMESIZE 128
#endif
