/*
    $Header: locking.c,v 1.5 90/02/22 19:36:57 gnu Exp $
    Copyright 1989 Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


    Lock some resource by name for gnuucp, if on a multitasking system.

*/

#ifndef lint
char locking_version[] = "$Header: locking.c,v 1.5 90/02/22 19:36:57 gnu Exp $";
#endif

#include "includes.h"
#include "uucp.h"
#include "sysdep.h"

extern int errno;

static char	lockname[NAMESIZE];	/* Name of lock file */
static int	lockfile;		/* File descriptor of lock file */

s_or_f uucpunlock();			/* Forward declaration */

/*
 * Lock a resource so that two programs won't try to mess with it
 * at the same time.
 *
 * On Unix machines, if you are using unix uucp as well as gnuucp,
 * these locks must be compatible with your unix uucp.
 *
 * Things that are locked include:
 *	tty names that we are dialing out on  ("cuaNNN")
 *	execution of queued jobs (X_LOCK, "XQT")
 *	accesses to the sequence file (SEQ_LOCK, "SEQF");
 *
 * You can only lock one thing at a time.  The next call to 
 * uucpunlock() will remove the lock.
 */
s_or_f
uucplock(name)
	char	*name;
{

#ifdef MULTITASK
	char newname[NAMESIZE];

	sprintf(newname, "LCK.%s", name);
	strcpy(lockname, munge_filename(newname));
	/* FIXME, next line is nonportable */
	lockfile = open(lockname, O_RDWR|O_BINARY|O_CREAT|O_EXCL, 0444);
	if (lockfile == -1) {
		DEBUG(9, "Can't create lock file %s\n", lockname);
		/* FIXME, check for expired lock (pid not running) here. */
		lockname[0] = '\0';	/* We don't hold a lock */
		return FAIL;
	}

	if (ourpid == 0)
		ourpid = getpid();

	if (sizeof ourpid != write(lockfile, (char *)&ourpid, sizeof ourpid)) {
		DEBUG(0, "Write of pid to %s failed\n", lockname);
		(void) close(lockfile);
		(void) uucpunlock();
		return FAIL;
	}
#endif

	return SUCCESS;
}

s_or_f
uucpunlock()
{

#ifdef MULTITASK
	if (lockname[0] == '\0')
		return SUCCESS;

	if (lockfile != -1 && 0 != close(lockfile)) {
		DEBUG(0, "Lock file close failed\n", 0);
	}
	lockfile = -1;

	if (0 != remove(lockname)) {
		DEBUG(0, "Removing lock file %s failed", lockname);
		DEBUG(0, " with errno %d\n", errno);
		lockname[0] = '\0';
		return FAIL;
	}
	lockname[0] = '\0';
#endif

	return SUCCESS;
}
