#include "includes.h"
#include "uucp.h"
#include "sysdep.h"

#define MAILCMD "rmail"
#define MAXTRIES 10

#ifdef VMS
#define TERMINATOR "CTRL-Z"
#define postmaster "system"
#define EXIT_OK 1
#define EXIT_ERR 0
#define TTY "SYS$INPUT:"
#define MAILSTR "MAIL %s %s"
#else
#define MAILSTR "/bin/mail < %s %s"
#define TERMINATOR "CTRL-D"
#include <sys/wait.h>
#define postmaster "postmaster"
#define EXIT_OK 0
#define EXIT_ERR 1
#define TTY "/dev/tty"
#endif

extern s_or_f read_params();
extern void logit();

#ifndef lint
char *version = "rmail VAX/VMS version 1.2";
#endif


main(argc, argv)
int argc;
char *argv[];
{
    int i;
    char rmtsite[80], rmtuser[80];
    char *argvuser = NULL;
    char *argvfile = NULL;
    int local = 0;
    int seq;
    FILE *fd, *tfd;
    char *cp, data[NAMESIZE], udata[NAMESIZE], datetime[40];
    char c, user[NAMESIZE];
    long temptime;
    extern int errno;

    for (i = 1; i < argc; i++) {
	if (argv[i][0] != '-')
	    break;
	switch (argv[i][1]) {

	default:
	    printf("rmail: unknown flag ignored\n");

	case 'x':
	    debug = atoi(&argv[i][2]);
	    printf("rmail: debug level set to %d\n", debug);
	    break;
	}
    }

    for (i = 1; i < argc; i++) {
	if (argv[i][0] == '-')
	    continue;

	if (argvuser == NULL) {
	    argvuser = argv[i];
	    continue;
	}
	if (argvuser != NULL) {
	    argvfile = argvuser;
	    argvuser = argv[i];
	    break;
	}
    }

    if (argvuser == NULL) {
	printf("usage: rmail [-xN] [file] \"site!site!...user[@site]\"\n");
	exit(EXIT_ERR);
    }

    read_params((char *)0);

    for (i = 0; i < argc; i++)
	DEBUG(1, "arg=%s\n", argv[i]);

    DEBUG(5, "user=%s\n", (argvuser == NULL ? "NULL" : argvuser));
    DEBUG(5, "file=%s\n", (argvfile == NULL ? "NULL" : argvfile));

    cuserid(who);		/* Finger out who we are */
    strcpy(host_name, "rmail");

    if (Spool == NULL) {
	printf("cannot read Spool directory from Control file\n");
	exit(EXIT_ERR);
    }
    if (Sysfile == NULL) {
	printf("cannot read Sysfile directory from Control file\n");
	exit(EXIT_ERR);
    }
    if (Myname[0] == NULL) {
	printf("cannot read Node name from Control file\n");
	exit(EXIT_ERR);
    }

    /* if you can't find a "!", assume local delivery */
    if ((i = mindex(argvuser, '!')) <= 0) {
	strcpy(rmtsite, "tmp");	/* VMS needs this or filenames aren't legal */
	local++;
    } else {
	strcpy(rmtsite, argvuser);
	rmtsite[i] = '\0';
	strcpy(rmtuser, &argvuser[i + 1]);
	validate_site(rmtsite);
    }

    if (argvfile == NULL) {
	argvfile = "stdin";
	fd = stdin;
    } else {
	if ((fd = fopen(argvfile, "r")) == NULL) {
	    logit("CAN'T OPEN", argvfile);
	    perror("can't open file for reading");
	    exit(EXIT_ERR);
	}
    }

/* FIXME FIXME FIXME FIXME FIXME FIXME:  all this should be ripped out
   and replaced by stuff that calls uux!  */

    if (chdir(Spool) != 0) {
	printf("cannot change to %s\n", Spool);
	perror("can't change directory");
	exit(EXIT_ERR);
    }
    seq = get_seq();
    sprintf(data, "D.%.7sA%04x", rmtsite, seq);
    strcpy(udata, munge_filename(data));	/* Turn to local form */

    DEBUG(1, "Using data name %s\n", udata);

#ifdef VMS
    if ((tfd = fopen(udata, "w", "rat = cr", "rfm = var")) == NULL) {
#else
    if ((tfd = fopen(udata, "wb")) == NULL) {
#endif
	logit("CAN'T CREATE", data);
	fprintf(stderr, "errno = %x\n", errno);
	perror("can't open file for writing");
	exit(EXIT_ERR);
    }
    time(&temptime);
    strcpy(datetime, ctime(&temptime));
    for (cp = datetime; *cp != '\n'; cp++);
    *cp = '\0';

    /* forward to rmtsite */
    cuserid(user);		/* who am I ? */
    for (cp = user; *cp; cp++)
	if (isupper(*cp))
	    *cp = tolower(*cp);

    fprintf(tfd, "From %s %s remote from %s\n", user, datetime, Myname);
    fprintf(tfd, "Received: by %s for %s on %s\n", Myname, argvuser, datetime);

    DEBUG(1, "reading %s\n", argvfile);

    /* copy data into spool file */
    while ((c = fgetc(fd)) != EOF)		/* FIXME, speed this up */
	fputc(c, tfd);

    DEBUG(1, "%s read\n", argvfile);

    fclose(fd);
    fclose(tfd);

    if (local)
	deliver(argvuser, data);
    else
	forward(rmtsite, rmtuser, data, user, seq);
    exit(0);
}

forward(rmtsite, rmtuser, dfile, user, seq)
char *rmtsite, *rmtuser, *dfile, *user;
int seq;

{
    FILE *fbfile, *fcfile;

    char bfile[NAMESIZE], cfile[NAMESIZE];
    char lbfile[NAMESIZE], lcfile[NAMESIZE];
    char rmtdfile[NAMESIZE], rmtxfile[NAMESIZE];
    char tmp[132];		/* FIXME */

    /* create a Bfile and Cfile. Dfile already exists  */

    sprintf(bfile, "B.%.7sA%04x", Myname, seq);
    sprintf(cfile, "C.%.7sA%04x", rmtsite, seq);

    sprintf(rmtxfile, "X.%.7sA%04x", Myname, seq);
    sprintf(rmtdfile, "D.%.7sA%04x", rmtsite, seq);

    /* The Bfile... */
    strcpy(lbfile, munge_filename(bfile));
    if ((fbfile = fopen(lbfile, "w")) == NULL) {
	perror("cannot open bfile for writing");
	exit(EXIT_ERR);
    }
    fprintf(fbfile, "U %s %s\n", user, Myname);
    fprintf(fbfile, "F %s\n", rmtdfile);
    fprintf(fbfile, "I %s\n", rmtdfile);

    fprintf(fbfile, "C %s %s\n", MAILCMD, rmtuser);

    if (fclose(fbfile) == EOF) {
	perror("cannot close cfile");
	exit(EXIT_ERR);
    }
    /* The Cfile... */

    strcpy(lcfile, munge_filename(cfile));
    if ((fcfile = fopen(lcfile, "w")) == NULL) {
	perror("cannot open cfile for writing");
	exit(EXIT_ERR);
    }
    fprintf(fcfile, "S %s %s %s - %s 0666\n", dfile, rmtdfile, user, dfile);
    fprintf(fcfile, "S %s %s %s - %s 0666\n", bfile, rmtxfile, user, bfile);

    if (fclose(fcfile) == EOF) {
	perror("cannot close fcfile");
	exit(EXIT_ERR);
    }
    sprintf(tmp, "C %s %s", MAILCMD, rmtuser);
    logit("XQT QUEUED", tmp);	/* FIXME, log filename(s) here so we can
				   trace a request from entry into the
				   uucp queue until it is transmitted. */
}

/* get a random number for names and other stuff */
random()
{
    int it;
    long tmp;

    it = time(&tmp);
    it &= 0xff;
    return (it);
}

/* return the index of c in st. The first char is at 0 */
mindex(st, c)
char c, st[];

{
    int i;

    for (i = 0; st[i] != '\0'; i++) {
	if (st[i] == c)
	    return (i);
    }
    return (-1);
}

validate_site(name)
char *name;

{
    char tmp[NAMESIZE];
    FILE *fd;
    int found = 0;

    if ((fd = fopen(Sysfile, "r")) == NULL) {
	printf("cannot open %s\n", Sysfile);
	logit("FAIL", "can't read Sysfile");
	exit(EXIT_ERR);
    }
    while (fgets(tmp, sizeof(tmp), fd) != NULL)
	if (strncmp(tmp, name, strlen(name)) == 0) {
	    found = 1;
	    break;
	}
    if (fclose(fd)) {
	printf("can't close %s\n", Sysfile);
    }
    if (!found) {
	printf("%s -- unknown site\n", name);
	logit("FAIL", "bad site");
	exit(EXIT_ERR);
    }
}

 /* Deliver file using local mailer If mail fails, send to it to postmaster.
  * If that fails, give up and die */

deliver(addressee, data)
char *addressee;
char *data;

{
    char line[NAMESIZE];
    char *temp_filename();

    /* Invoke the mailer */
    if (mail(addressee, data)) {
	sprintf(line, "%s %s %s %s", "mail", addressee, data);
	logit("FAILED", line);
	if (mail(postmaster, data)) {
	    sprintf(line, "%s %s %s %s", "mail", postmaster, data);
	    logit("FAILED", line);
	    exit(EXIT_ERR);
	}
    }
    sprintf(line, "%s %s %s", "mail", addressee, data);
    logit("OK", line);
    if (remove(data) == 0)
	logit("deleted", data);
    else
	logit("Can't delete", data);
}

mail(addressee, input)
char *addressee, *input;

{
    char line[255];
    int status;

    sprintf(line, MAILSTR, input, addressee);
    DEBUG(4, "calling system: %s\n", line);
    status = system(line);
    DEBUG(4, "system returned status = %d\n", status);
    return status;
}

/*
 * pick a random num and make a unique file name from it
 *
 * FIXME: How do real systems do this? What about ensuring uniqueness
 * at the remote site?
 */
get_seq()
{
    int i;
    int seq;
    char cfile[40];

    for (i = 0; i <= MAXTRIES; i++) {
	seq = random();
	sprintf(cfile, "C.%.7sA%04x", Myname, seq);
	if (access(cfile, 0) != 0)	/* doesn't already exist */
	    return seq;
	if (i == MAXTRIES) {
	    printf("can't generate unique file name\n");
	    exit(EXIT_ERR);
	}
    }
}
