/*
    $Header: sysdep.st,v 1.6 89/04/26 21:26:51 gnu Exp $
    Copyright 1989 Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    sysdep.st for use with MT C-Shell on Atari ST
*/

#ifndef lint
char sysdep_version[] = "$Header: sysdep.st,v 1.6 89/04/26 21:26:51 gnu Exp $";
#endif

#include "includes.h"
#include "uucp.h"
#include "sysdep.h"

#include "stat.h"	/* FIXME, what is this for? -- hoptoad!gnu */

#ifdef RTX
#include <rtxbind.h>
#include "termio.h"
#endif


char sysdep_control[] = "\\usr\\spool\\uucp\\gnuucp.ctl";

/* Set up a 16K stack for MWC */
long _stksize = 16384;
extern char **environ;
extern int errno;

#define AUX		1	/* rs232 port */
#define CON		2	/* console */
#define CTRL(X)	(X & 037)

int xdev = AUX;			/* default device is AUX */

/* Our variables */
void sigint();			/* Forward declaration */


/*
 * Open the serial line for an incoming call.
 * Argument of NULL or empty string means stdin.
 */
int
openline(ttynam, baud)
	char	*ttynam;
	long	baud;
{
	return(opentty(ttynam, baud));
}

/*
 * Basement level I/O routines
 *
 * xwrite() writes a character string to the serial port
 * xgetc() returns a character from the serial port, or an EOF for timeout.
 * sigint() restores the state of the serial port on exit.
 */

void
sigint()
{
	/* Restore terminal settings on dialout line */

	/* No need to do anything here? */
	exit(0);
}


/*
 * xwrite(buf, n) - write "n" bytes from buffer "buf" to rs232 port.
 */
xwrite(buf, cnt)
	int cnt;
	char *buf;
{
	register char *c = buf;
	register int n = cnt;

	while (n) {
		/* wait for rs232 to be ready */
		while (Bcostat(xdev) == 0)
			;

		/* write next character in buffer */
		Bconout(xdev, *c++);
		n--;
	}
	return(cnt);
}

/*
 * Serial port reading routines
 */

/*
 * Atari ST routines for reading the comm port.
 *
 * The following routines come by way of J. R. Bammi
 */

long prtime = 0L;			/* present time			*/
long alrmtime = 0L;			/* alarm time			*/
long *hz200 = (long *)0x0004ba;		/* address of system 200 hz clk	*/

/*
 * read200hz() - read the system 200 hz clock.
 */
void read200hz()
{
	char *ssp;

	ssp = Super(0L);
	prtime = *hz200;
	Super(ssp);
}

/*
 * alarm(n) - set the alarm time to n seconds.
 */
void alarm(n)
	unsigned int n;
{
	/* if n != 0, then set the alarm time */
	if (n) {
		read200hz();
		alrmtime = prtime + (long)(200 * n);

	/* n == 0, reset alarm time */
	} else
		alrmtime = 0L;
}

/*
 * xgetc() - get a character from the rs232 port, catch timeouts & aborts.
 */
xgetc()
{
	alarm(BYTE_TIMEOUT);
	while (1) {
#ifdef DEBUG_SYSDEP
		/* catch ^C (user abort) */
		if (Bconstat(CON))
			if ((int)Bconin(CON) == CTRL('C'))
				exit(1);
#endif
#ifdef RTX_DEBUG
 Bconout(CON, '+');
#endif
		/* check for a char at rs232 port */
		if (Bconstat(xdev)) {
			alarm(0);
#ifdef RTX_DEBUG
 Bconout(CON, '-');
#endif
			return((int)Bconin(xdev));

		/* no char, check for timeout */
		} else if (alrmtime) {
			read200hz();
			if (prtime >= alrmtime) {
#ifdef RTX_DEBUG
 Bconout(CON, '!');
#endif
				return(EOF);
			}
		}
#ifdef RTX
		p_pause(50L);
#endif
	}
}

/* ST library needs these routines.  Probably should use
 * the new names, but for now...
 */
bzero(s, cnt)
register char	*s;
register int	cnt;
{
	register int	i;
	for (i = 0; i < cnt; i++) {
		*s++ = '\0';
	}
}

bcopy(from, to, cnt)
register char	*from;
register char	*to;
register int	cnt;
{
	register int	i;
	for (i = 0; i < cnt; i++) {
		*to++ = *from++;
	}
}

/*
 * Create a temporary file name for receiving a file into.
 * "name" is the name we will actually eventually want to use for the file.
 * We currently ignore it, but some OS's that can't move files around
 * easily might want to e.g. put the temp file into the same directory
 * that this file is going into.
 *
 * FIXME:
 * This interface should be able to return a "possible" filename, and
 * be re-called if the name is already in use, to get another.
 * This avoids checking here whether the name is good -- saving system calls.
 */
char *
temp_filename(name)
	register char *name;
{
	static char tname[NAMESIZE];
	char *t;

	sprintf(tname, "%s\\%s\\uut%03d.$$$", Spool, host_name, getpid());

#ifdef DEBUG_SYSDEP
	printf("temp_filename: '%s'\n", tname);
#endif
	return tname;
}


/*
 * Transform a filename from a uucp packet (in Unix format) into a local
 * filename that will work in the local file system.
 *
 * FIXME:  this is not done for anything but C. and D. files currently.
 *         It does NOT work for LCK. and STST. files!
 */
char *
munge_filename(name)
	register char *name;
{
	register char *p;

	static char buffer[NAMESIZE+SLOP];
	char mname[MAX_HOST];		/* master's name */
	char cwdir[NAMESIZE];		/* current working director */
	char ufname[WSUFSIZE + 1];	/* distinctive 5 char UU file ID */
	int i;				/* counter */

#ifdef DEBUG_SYSDEP
	printf("Munge_filename  input: %s\n", name);
#endif
	if (name[0] == '~') {
		if (name[1] == '/')
			/* convert ~/name to PUBDIR/name */
			sprintf(buffer, "%s\\%s", Pubdir, name+2);
		else
			/* convert ~name to $HOME/name */
			sprintf(buffer, "\\usr\\%s", name+1);
	}
	else if (name[1] != '.') {
		/* not a UUCP internal file so hope it will work */
		strcpy(buffer, name);	
	} else {
		/*
		 * This is a uucp-internal C., D., or other name.
		 * Munge the hell out of it.  For MSDOS, we can't
		 * handle the 14-character names of Unix, so change
		 * it to:
		 * 
		 * L.hostGNNNN  -> host/GNNNN.L
		 */
		bzero(cwdir, sizeof cwdir);
		bzero(ufname, sizeof ufname);

		i = strlen(name);
		strncpy(mname, &name[2], i - (2 + WSUFSIZE));
		mname[i - (2 + WSUFSIZE)] = '\0';
		strcpy(ufname, &name[i - WSUFSIZE]);

		strcpy(buffer, Spool);
		strcat(buffer, "\\");
		strcat(buffer, mname);	/* get host's directory name */

		/*
		 * Quick (?) test to see if we already have a directory
		 * for this host.  FIXME, this looks really slow to me.
		 */
		Dgetpath(cwdir,0);		/* save current directory */
		if (chdir(buffer) == -1) {	/* If new dir doesn't exist, */
			if (mkdir(buffer) == -1) {	/* ... make one */
#ifdef DEBUG_SYSDEP
 printf("Can't make new directory %s\n", buffer);
#endif
			} else {
#ifdef DEBUG_SYSDEP
 printf("Making new directory %s\n", buffer);
#endif
			}
		} else
			chdir(cwdir);	/* pop back to the home directory */
		
		/* build actual file name */
		sprintf(buffer+strlen(buffer), "\\%s.%c", ufname, name[0]);
	}

	/* finish off by converting any remaining '/'s to '\' for TOS */
	for (p = buffer; *p; p++)
		if (*p == '/')
			*p = '\\';
	/* hope that we won't munge it anymore */
#ifdef DEBUG_SYSDEP
	printf("Munge_filename output: %s\n", buffer);
#endif
	return buffer;
}



/*
 * Uucp work queue scan.  For calling sequences, see sysdep.h.
 */
static DMABUFFER	xfbuf;
static int	workfile;		/* Do we have one in xfbuf? */
static char	workhost[MAX_HOST+1];
static int	workhlen;
static char	workfirst = 0;
extern char	tempnam[MAX_HOST + 30 + SLOP];


/*
 * Returns 1 if a matching file was found, 0 if not.
 */
find(pathname,flag,pxfbuf)
	char *pathname;
	int flag;
	DMABUFFER *pxfbuf;
{
	int x;

	Fsetdta(pxfbuf);
	if (flag == 0) 
		  x = Fsfirst(pathname, 0x3f);
		/* FIXME, 0x3F should be a #define! */
	else {
		  x = Fsnext();
		  flag = 1;
	}
#ifdef DEBUG_SYSDEP
 printf("scanning for %s, found '%s'\n", pathname, 
  x == 0 ? pxfbuf->d_fname : "<null>");
#endif
 	return(x == 0);
}
void
work_done()
{
	workfile = 0;
	workfirst = 0;
}
int
work_scan(host, type)
	char *host;
	char *type;		/* E.g. "C", "X", "LCK" */
{

	/* Null argument produces all hostnames */
	if (!host || !*host) host="*";

	/* If called twice in a row, don't thrash the disk again */
	if (strcmp(workhost, host) == SAME && workfile)
		return 1;

	workfirst = 1;				/* Initialize for work_next */
	workhlen = strlen(host);
	if (workhlen > sizeof (workhost) -1) abort();
	strcpy(workhost, host);
	if (workhlen > 7) workhlen = 7;		/* Unix uucp limit */

	strncpy(tempnam, workhost, workhlen);
	strcpy(tempnam+workhlen, "\\*.");
	strcat(tempnam+workhlen+    3, type);

	workfile = find(tempnam, 0, &xfbuf);
	if (!workfile) {
		DEBUG(8, "work_scan readdir null\n", 0);
		work_done();
		return 0;		/* No work */
	}
	return 1;
}


static int
work_look()
{
	int len;

	for (;;) {
		workfile = find(tempnam, 1, &xfbuf);
		if (!workfile) {
			DEBUG(8, "work_look readdir null\n", 0);
			work_done();
			return 0;		/* No work */
		}
		DEBUG(8, "work_look readdir %s\n", xfbuf.d_fname);
		/* FIXME, check grade letter! */
		return 1;	/* Found work */
	}
	/* NOTREACHED */
}


char *
work_next()
{
	char *p;

	/* Space for:	   "TYPE.hostnameA1234" */
	static char subhack[6    +MAX_HOST + WSUFSIZE + SLOP];

	if (!workfirst) {
		if (!workfile || !work_look())
			return (char *)NULL;
	}
	workfirst = 0;

	/* FIXME, This probably fails for STST and LCK files. */
	p = index(xfbuf.d_fname, '.');
	if (!p) abort();	/* There must be a filename extension */
	*p = '\0';
	sprintf(subhack, "%s.%s%s", p+1, workhost, xfbuf.d_fname);
	return subhack;
}


/* FIXME: fake/unfinished routines */

static char lockname[NAMESIZE];

ttyunlock()
{
#ifdef RTX
	if (lockname[0] == '\0')
		return SUCCESS;

	if (0 != remove(lockname)) {
		DEBUG(0, "Removing lock file %s failed", lockname);
		DEBUG(0, " with errno %d\n", errno);
		lockname[0] = '\0';
		return FAIL;
	}
	lockname[0] = '\0';
#endif
	return SUCCESS;
}
hangup()
{
	ttyunlock("tty");
}


/*
 * Routine to return a string that gives the current date and time, and
 * identifies the current process, if on a multiprocess system.
 */
char *
time_and_pid()
{
	time_t t;
	tm_t *tm;
#ifdef RTX
	static char format[] = "%03d-%d/%d-%02d:%02d:%02d";
#else
	static char format[] = "%d/%d-%02d:%02d:%02d";
#endif
	static char tbuf[sizeof(format)];

	time(&t);
	tm = localtime(&t);
	sprintf(tbuf, format,
#ifdef RTX
		getpid(),	/* Process ID, if multiuser */
#endif
		tm->tm_mon+1, tm->tm_mday,
		tm->tm_hour, tm->tm_min, tm->tm_sec);	

	return(tbuf);
}

rename(f1, f2)
char *f1, *f2;
{
	return(Frename(0, f1, f2));
}

remove(f1)
char *f1;
{
	return(Fdelete(f1));
}

int bdtbl[] = {
	19200,
	9600,
	4800,
	3600,
	2400,
	2000,
	1800,
	1200,
	600,
	300,
	200,
	150,
	134,
	110,
	75,
	50,
	0
};

/* internal routine used by openline() and openout() */
opentty(ttynam, baud)
char *ttynam;
long baud;
{
	int lockfile, ourpid, pid;
	int fd, *b;
	struct termio termio;
	int xbaud;

#ifdef RTX
	if ((fd = Fopen(ttynam, 2)) < 0) {
		DEBUG(9, "cannot open %s\n", ttynam);
		return(-1);
	}
	xdev = Ftype(fd);
	Fclose(fd);
	if (xdev < 0 || xdev >= 16) {
		DEBUG(9, "%s: not a tty device\n", ttynam);
		return(-1);
	}
	/* set the baud rate */
	for (xbaud = 0, b = bdtbl; *b; b++, xbaud++)
		if (*b == baud) {
		termio.t_cmd = TTY_CONF;
		termio.t_baud = xbaud;
		termio.t_mask = -1;
		termio.t_flags = 0;
		d_cntrl(xdev, &termio);
		break;
	}
	if (*b == 0 && DEBUG_LEVEL(9)) {
		printf("bad baud rate: %d\n", baud);
		return(-1);
	}
	/* FIXME: I put tty locking here? */
	sprintf(lockname, "%s.lck", ttynam);
	lockfile = open(lockname, 0);
	ourpid = getpid();
	if (lockfile > 0) {
		/* check to see if we already have the TTY locked */
		if (sizeof pid == read(lockfile, (char *)&pid, sizeof pid)) {
			if (pid == getpid()) {
				close(lockfile);
				return(0);
			}
		}
		/* tty is locked by some other process */
		DEBUG(9, "tty lock %s already exists\n", lockname);
		/* FIXME, check for expired lock (pid not running) here. */
		lockname[0] = '\0';	/* We don't hold a lock */
		close(lockfile);
		return FAIL;
	}
	lockfile = creat(lockname, 0644);	
	if (lockfile < 0) {
		DEBUG(9, "Can't create tty lock %s\n", lockname);
		lockname[0] = '\0';	/* We don't hold a lock */
	}
	if (sizeof ourpid != write(lockfile, (char *)&ourpid, sizeof ourpid)) {
		DEBUG(0, "Write of pid to %s failed\n", lockname);
		(void) close(lockfile);
		ttyunlock(ttynam);
		return FAIL;
	}
	close(lockfile);
#endif	
	return(0);
}

openout(ttynam, baud)
char *ttynam;
long baud;
{
	return(opentty(ttynam, baud));
}

exec_uux()
{
	int err;
	char *argv[2];

	argv[0] = "uuxqt";
	argv[1] = 0;

	err = execve("\\usr\\bin\\uuxqt.prg", argv, environ);
	if (err && DEBUG_LEVEL(2))
		printf("exec uuxqt %d (errno=%d)\n", err, errno);
	return(err);
}


/* FIXME: UNIX(tm) routines not in ST libs */

static char *old;

char *strtok(s1, s2)
char *s1, *s2;
{
	char *s;

	if (s1 == 0)
		s1 = old;
	while (*s1 && strchr(s2, *s1))
		s1++;
	s = s1;
	while (*s1 && !strchr(s2, *s1))
		s1++;
	if (s == s1)
		return(0);
	old = s1;
	if (*old)
		old++;
	*s1 = 0;
	return(s);
}
setuid()
{
}
getuid()
{
	return(0);
}
rtx_hack()		/* FIXME: run uucico non-slicable */
{
#ifdef RTX
	p_priority(0L, 255 - p_priority(0L, 0));
#endif
}

/* RTX conditionals */
#ifndef RTX
cuserid(s)
char *s;
{
	strcpy(s, "root");
}
getpid()
{
	return(1);
}
#endif
