/*
Date: Mon, 6 Mar 89 13:24:34 EST
From: Simmule Turner <simmy@ocean.ocean.fsu.edu>
Message-Id: <8903061824.AA03974@ocean.ocean.fsu.edu>
To: hoptoad!gnu
Subject: Simple uucp command
*/

/*
 * Simple minded uucp - driver for gnuucp.
 * Copyright 1989 Simmule Turner 03/05/89 */

#include "includes.h"
#include "uucp.h"
#include "sysdep.h"

/* Comment out next, line to allow transfers to transfer
 * anywhere within file tree uucp has write permissions. */
#define SECURE  /* Only allow transfers to/from ~uucp directory */

#ifdef SECURE
#define SECURITY(x) some(x)
	char       *some();
#else
#define SECURITY(x) none(x)
	char       *none();
#endif

#define MAXTRIES        10
#define TOLOCAL         0
#define TOREMOTE        1

char *username;

extern s_or_f read_params();
extern void logit();
extern char *getseqnum();


main(argc, argv)
int argc;
char *argv[];
{
   char local_file[NAMESIZE] ,
        remote_file[NAMESIZE],   *remote_name,
        *p0, *p1, *cuserid(), *index(), flag = 0;

   if (argc != 3)
     usage();

   /* We should be checking here for index(,'!') < index(,'/') to allow
      filenames containing ! to be moved. */
   p0 = index(argv[1],'!');
   p1 = index(argv[2],'!');

   {
      int temp = (p0 ? 1 : 0) + (p1 ? 1 : 0);

      if (temp != 1)
        usage();
   }

   username = cuserid((char *) NULL);
   if (read_params((char *) 0) != SUCCESS)
	exit(1);

   if (p0)
     *p0++ = 0;
   else
     {
        *p1++ = 0;
        flag++;
     }

/* FIXME:  check for multiple hops, either don't allow them,
 *         or setup a uux command to handle it. */

   remote_name = argv[flag + 1];
   strcpy(remote_file, flag ? p1 : p0);
   strcpy(local_file, argv[flag ? 1 : 2]);
   copy(remote_file,remote_name,local_file, flag ? TOREMOTE : TOLOCAL);
}

copy(rf,rn,lf,flag)
char *rf, *rn, *lf;
int flag;
{
   FILE *fp;
   char cfile[20], lcfile[NAMESIZE], str[500];

/*
   We should be checking that RN is a valid remote site in our sys file.
 */

   sprintf(cfile,"C.%.7sn%s", rn, getseqnum());
   strcpy(lcfile,munge_filename(cfile));

   if (flag == TOLOCAL)
     sprintf(str,"R %s %s %s -dc\n",rf,SECURITY(lf),username);
   else
     sprintf(str,"S %s %s %s -dc D.0 0666\n",SECURITY(lf),rf,username);

   if (chdir(Spool) != 0)
     {
        printf("Cannot change to %s\n",Spool);
        perror("Can't change directory");
        exit(1);
     }

   if ((fp = fopen(lcfile,"w")) == (FILE *) NULL)
     {
	/* This failure might be because a directory has not yet been
	   created for the queue file.  Hmm....maybe the code that creates these
	   directories needs to go into sysdep too?  FIXME */
        perror("Cannot open cfile for writing");
        exit(1);
     }

   fputs(str, fp); 
   if (fclose(fp) == EOF)
     {
        perror("Cannot close cfile");
        exit(1);
     }

   str[strlen(str)-1] = 0;
   logit("XQT QUEUED",str);
}

usage()
{
   printf("usage:  uucp localfile site!remotefile\n");
   printf("        uucp site!remotefile localfile\n");
   exit(1);
}

static char u_buf[NAMESIZE];

#ifdef SECURE
#define IN(a,b) (strncmp(a,b,strlen(a)) == 0)

char *
some(s)
char *s;
{

   if (IN(s,"~/") ||
       IN(s,"~uucp/") ||
       IN(s,"/usr/spool/uucppublic"))
     return(s);
   else
     {
        char *p=index(s,'/');

        if (p)
          {
             printf("Transfers only allowed to/from /usr/spool/uucppublic\n");
             exit(1);
          }

        strcpy(u_buf,"~/");
        strcat(u_buf,s);
        return(u_buf);
     }
}
        
#else
char *
none(s)
char *s;
{
   if (*s == '/')
     return(s);
 
   getcwd(u_buf,NAMESIZE);
   strcat(u_buf,"/");
   strcat(u_buf,s);
   return(u_buf);
}
#endif
/*
--
Simmule Turner        |Arpa: simmy@nu.cs.fsu.edu        | "Wait until it is 
Florida State Univ    |Uucp: gatech!simmy@nu.cs.fsu.edu |  finished, it will
444 OSB               | Cis: 70651,67       Genie:simmy |  be great"        
Tallahassee, FL 32306 | Tel: +1 904 644 1573            |             simmy
*/
