/*
    $Header: uucp.h,v 1.16 90/02/24 00:11:25 gnu Exp $
    Copyright 1989 Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


    Header file for gnuucp.

    Pieces that look like they might be taken from Unix uucp are
    cribbed with the aid of public domain uucp modules (like the F-protocol)
    that use that interface.  No AT&T-licensed sources were used in building
    gnuucp.
*/

#define Ifn	FOO	/* file descriptor of the "phone line" */

#define	ASSERT(cond, msg, moremsg, huh) /**/
#define	prefix(shrt, lng)	(strncmp(shrt, lng, strlen(shrt)) == SAME)

/*
 * Control debugging messages.
 */
#define	DEBUG(level, msg, moremsg) \
	if (debug >= level) printf(msg, moremsg)
#define	DEBUG_LEVEL(level) \
	   (debug >= level)
#define DEBUG_DATA(data) \
	(*Debug_data)(data)
	
/* I may have these flipped from what Unix uses... */
/* BUT FIXME: my code depends on these values in if (foo()) statements, sigh */
typedef enum s_or_f { SUCCESS = 0, FAIL = 1 } s_or_f;
/* If your compiler does not support enums well, you can do the following:
	#define s_or_f	int
	#define	SUCCESS	0
	#define	FAIL	1
 */

#define	MAXMSGLEN	((NAMESIZE*4)+SLOP)	/* ?>?? FIXME */
#define	MAXMSGTIME	45	/* Timeout period for rdmsg */

#define	MAXBASENAME	7	/* Host name length in queue file names */
#define	WSUFSIZE	5	/* Queued work file suffix (grade+seq#) */
#define	SYSNSIZE	MAXBASENAME	/* uuq uses */
#define	MAX_QFNAME	MAXBASENAME+WSUFSIZE+2+1 /* Max queue file name len */
				/* +2 for "C." etc; +1 for null */

#define	CMDPRE		'C'	/* Prefix on command files */
#define	DEF_GRADE	'A'	/* default job grade */

#define	X_LOCK		"XQT"	/* Lock name for uuxqt runs */
#define	SEQ_LOCK	"SEQL"	/* Lock name for sequence numbers */

#define	MAX_HOST	20	/* Host name length (unix uucp does 7) */
#define	MAX_LSYS	500	/* Max length of an L.sys line in chars */
#define	MAX_CTLLINE	100	/* Max length of a gnuucp.ctl line */

#define	CTL_DELIM	" \t\n\r"	/* Delimiters for gnuucp.ctl */

extern int debug;			/* Debugging level */

/*
 * Timeout for raw characters -- if we don't hear a char within BYTE_TIMEOUT
 * seconds, we assume the other side has gone away.  Has nothing to do with
 * retransmission timeouts (if any!).
 */
#define	BYTE_TIMEOUT	45		/* seconds */
#define	ONEDAY		(24*60*60)	/* seconds */

/* Misc stuff */
#define	SAME		0		/* if (strcmp(a,b) == SAME) ... */
#define	SLOP		10		/* Slop space on arrays */

/* STST system status file codes */
#define	SS_WRONGTIME	99		/* Wrong time to call -- not used */
#define	SS_INPROGRESS	3		/* Talking */
#define SS_FAIL		4		/* Dial failed */
#define	MAXRECALLS	20		/* Max # of recalls if STST there */

/*
 * Definition of serial ports that we can call out on.
 */
#define	PORTNAME_SIZE	20		/* Arbitrary */
struct port {
	char	portname[PORTNAME_SIZE]; /* Name of port type, e.g. ACU */
	char	modemname[PORTNAME_SIZE]; /* Name of dialer type, e.g. hayes */
	char	devname [PORTNAME_SIZE]; /* Name of device, e.g. cua0, com1 */
	long	baud;			/* A baud rate that works */
	struct port *chain;		/* Chain to next entry */
};

/*
 * Table of words recognized in L.sys time specifications.
 * This table can be extended at runtime in the gnuucp control file.
 * This table should be referenced via global pointer variable Timetable.
 */
struct timetable {
	char	*str;		/* "Su", "Wk", etc. */
	enum {Day, Recurse, Any} what;	/* How to process it */
	int	dayval;		/* What day of the week is represented */
	struct timetable *next;	/* Next timetable entry to use */
	char	*recurseval;	/* String to recurse with */
};

/* Variables that uucp uses for various things */
extern char	*Spool;			/* Our spool directory */
extern char	*Control;		/* Our control file */
extern char	Myname[MAX_HOST];	/* Our host name */
extern char	host_name[MAX_HOST];	/* Other guy's host name */
extern int	ourpid;			/* Our process ID */
extern char	*Sysfile;		/* L.sys file */
extern char	*Userfile;		/* USERFILE */
extern char	*Pubdir;		/* Public Directory */
extern struct port *ports;		/* List of serial ports */
extern char	who[NAMESIZE];		/* User who sent/requested a file */
extern int	logfd;			/* File descriptor of log file */
extern char	*Gnuuio;		/* Program name to run for gnuuio */
extern char	*Gnuuxqt;		/* Program name to run for gnuuxqt */
extern struct timetable *Timetable;	/* Table of time names for L.sys */
extern void	(*Debug_data)();	/* Fn to print serial line data */
extern char	*Seqfile;		/* Holds last used sequence number */
extern char	*Lockdir;		/* Dir holding lock files */
extern char	*log_sequence;		/* Ptr to seq string to log, or null */
