/*
    $Header: uumisc.c,v 1.17 90/08/30 22:44:49 gnu Exp $
    Copyright 1989 Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


    uumisc.c

    Miscellaneous uucp variables and routines which are used by several
    uucp main programs.
*/
#include "includes.h"		/* System include files, system dependent */
#include "uucp.h"		/* Uucp definitions and parameters */
#include "sysdep.h"		/* System dependent parts of gnuucp */

extern s_or_f	uucplock();
extern s_or_f	uucpunlock();

/*
 * Table of words recognized in L.sys time specifications.
 * This table can be extended at runtime by the Control file.
 */
#define TTD timetable_default
struct timetable timetable_default[] = {
	{"su",	Day,		0,	&TTD[1],	(char	*)0},
	{"mo",	Day,		1,	&TTD[2],	(char	*)0},
	{"tu",	Day,		2,	&TTD[3],	(char	*)0},
	{"we",	Day,		3,	&TTD[4],	(char	*)0},
	{"th",	Day,		4,	&TTD[5],	(char	*)0},
	{"fr",	Day,		5,	&TTD[6],	(char	*)0},
	{"sa",	Day,		6,	&TTD[7],	(char	*)0},
	{"any",	Any,		0,	&TTD[8],	(char	*)0},
	{"wk",	Recurse,	0,	&TTD[9],	"Mo,Tu,We,Th,Fr"},
	{"evening", Recurse,	0,	&TTD[10],	"Wk1700-0800,Sa,Su"},
	{"night", Recurse,	0,	&TTD[11],
					      "Any2300-0800,Sa,Su0000-1700"},
	{"nonpeak", Recurse,	0,	(struct timetable *)0,	/* End table */
							"Any1800-0700,Sa,Su"},
};

/* The following declarations should exactly match those in uucp.h */
char	Myname[MAX_HOST];	/* Our uucp hostname, set from gnuucp.ctl */
char	host_name[MAX_HOST] = "gnuucp";	/* Other guy's host name */
char	*Control;		/* Control file name */
char	*Spool;			/* Spool directory root name */
char	*Sysfile;		/* L.sys file */
char	*Userfile;		/* USERFILE */
char	*Pubdir;		/* Public Directory */
char	*Logfile;		/* Logging file */
char	who[NAMESIZE] = "-";	/* Who sent us a file */
int	debug = -1;		/* -1 indicates ctl file should set it */
int	ourpid = 0;		/* Our process ID */
int	logfd;			/* file desc of uucp logfile */
char	*Gnuuio;		/* Program name to run for gnuuio */
char	*Gnuuxqt;		/* Program name to run for gnuuxqt */
struct timetable *Timetable = timetable_default; /* Table of time names */
struct port *ports;		/* List of serial ports */
void	(*Debug_data)();	/* Fn to display serial line data */
char	*Seqfile;		/* Holds last used sequence number */
char	*Lockdir;		/* Dir holding lock files */
char	*log_sequence;		/* Ptr to sequence string, or null */

struct port *porttail;		/* End of chain of serial port definitions */

char	*gimmestring();		/* Forward declaration */

/*
 * Read the control file and grab a few parameters.
 */
s_or_f
read_params(ctl)
	char *ctl;
{
	FILE	*fd;
	char	buf[MAX_CTLLINE];
	register char *cmd;
	register char	*p;

	if (ctl) {
		/* Remove any possible setuid if user spec'd a control file */
		/* FIXME: this is system dependent....sigh */
#ifndef MSDOS
		setuid(getuid());
#endif
	} else {
		/* Use the system default control file */
		ctl = sysdep_control;
	}
	Control = ctl;
	if (! (fd = fopen(ctl, "r"))) {
		fprintf(stderr, "Can't find configuration file %s\n", ctl);
		return FAIL;
	}

	/* Scan the file, parse out various settings. */
	while (NULL != fgets(buf, sizeof buf, fd)) {

		cmd = strtok(buf, CTL_DELIM);
		if (!cmd) continue;		/* Skip blank lines */
		for (p = cmd; *p; p++)
			if (isupper(*p)) *p = tolower(*p);

		/* Now figure out which one it is */

		       if (strcmp(cmd, "spool") == 0) {
			Spool = gimmestring();
		} else if (strcmp(cmd, "sysfile") == 0) {
			Sysfile = gimmestring();
		} else if (strcmp(cmd, "userfile") == 0) {
			Userfile = gimmestring();
		} else if (strcmp(cmd, "logfile") == 0) {
			Logfile = gimmestring();
		} else if (strcmp(cmd, "seqfile") == 0) {
			Seqfile = gimmestring();
		} else if (strcmp(cmd, "lockdir") == 0) {
			Lockdir = gimmestring();
		} else if (strcmp(cmd, "pubdir") == 0) {
			Pubdir = gimmestring();
		} else if (strcmp(cmd, "gnuuio") == 0) {
			Gnuuio = gimmestring();
		} else if (strcmp(cmd, "gnuuxqt") == 0) {
			Gnuuxqt = gimmestring();
		} else if (strcmp(cmd, "nodename") == 0) {
			strcpy(Myname, strtok((char *)NULL, CTL_DELIM) ) ;
		} else if (strcmp(cmd, "debug") == 0) {
			if (debug < 0) {
				debug = atoi(strtok((char *)NULL, CTL_DELIM));
			}
		} else if (strcmp(cmd, "port") == 0) {
			register struct port *newport;

			newport = (struct port *)malloc(sizeof *newport);
			p = strtok((char *)NULL, CTL_DELIM);
			strcpy(newport->portname, p);
			p = strtok((char *)NULL, CTL_DELIM);
			strcpy(newport->modemname, p);
			p = strtok((char *)NULL, CTL_DELIM);
			strcpy(newport->devname, p);
			p = strtok((char *)NULL, CTL_DELIM);
			newport->baud = atol(p);
			/* Add it to the end of the chain */
			newport->chain = (struct port *)0;
			if (ports)
				porttail->chain = newport;
			else
				ports = newport;
			porttail = newport;
		} else if (strcmp(cmd, "timetable") == 0) {
			register struct timetable *newtime;
			register char *newname;

			newtime = (struct timetable *)malloc(sizeof *newtime);

			p = strtok((char *)NULL, CTL_DELIM);
			newname = (char *)malloc ((unsigned)strlen(p)+1);
			newtime->str = newname;
			while (*p) {
				if (isupper(*p))
					*newname++ = tolower(*p);
				else
					*newname++ = *p;
				p++;
			}
			*newname = '\0';	/* Terminate it */

			newtime->what = Recurse;
			newtime->dayval = 0;

			p = strtok((char *)NULL, CTL_DELIM);
			newname = (char *)malloc ((unsigned)strlen(p)+1);
			newtime->recurseval = newname;
			strcpy(newname, p);

			newtime->next = Timetable;
			Timetable = newtime;
		}

		/* Unrecognized lines are deliberately ignored.  This
		   control file can also be used by other programs, e.g.
		   for netnews or email processing. */
	}

	fclose(fd);

	/* Defaults that must be defaulted at runtime go here */

	if (Lockdir == 0)	Lockdir = Spool;

	return SUCCESS;
}

/* Scan out next arg as a string, return it or NULL if none there. */
char *
gimmestring()
{
	register char *p, *q;

	p = strtok((char *)NULL, CTL_DELIM);
	if (p == NULL)
		return p;
	q = (char *)malloc((unsigned)strlen(p)+1);
	strcpy(q, p);
	return q;
}

/*
 * Scan out an "argv" from a blank-separated string.
 *
 * Result is argc, or -1 if there are too many args.
 */
int
getargs(string, argv, maxargc)
	char	*string;
	char	**argv;
	int	maxargc;
{
	int	argc = 0;

	while (*string == ' ') string++;

	for (; *string; ) {
		argv[argc++] = string;		/* Record position */
		if (argc >= maxargc) return -1;
		while (*string && *string != ' ') string++;
		if (*string == '\0') break;
		*string++ = '\0';		/* Null terminate it */
		while (*string == ' ') string++;
	}

	argv[argc] = (char *)0;			/* Null after last one */
	return argc;
}

/*
 * Log file writing subroutine.
 *
 * Makes incredibly ugly log entries that look *just like* Unix uucp's
 * incredibly ugly log entries.
 *
 * Once we don't care about compatability, we should do this much better.
 */
void
logit(one, two)
	char *one, *two;
{
	char logbuf[(NAMESIZE*4)+SLOP+50];	/* Temp buffer for logs */
	char seqnum[WSUFSIZE+1];
	int len;

	if (logfd <= 0) {
		/* Open the log file if not already open */
		if (Logfile == 0 || *Logfile == 0)
			return;		/* If none specified, no logging. */
		logfd = open(Logfile, O_CREAT|O_WRONLY|O_APPEND, 0644);
#if O_CREAT == 0
		if (logfd < 0)
			logfd = creat(Logfile, 0644);
#endif
		if (logfd < 0) {
			if (DEBUG_LEVEL(0)) perror(Logfile);
			exit(39);
		}
#if O_APPEND == 0
		lseek(logfd, 0L, SEEK_END);	/* Seek to eof */
#endif
	}

	/* If the logging sequence number is set, append it to the log */
	if (log_sequence == 0)
		seqnum[0] = '\0';
	else
		sprintf(seqnum, "-%s", log_sequence);

	sprintf(logbuf, "%s %s (%s%s) %s (%s)\n",
		who, host_name, time_and_pid(), seqnum, one, two);

	DEBUG(0, "%s", logbuf);

	len = strlen(logbuf);
	if (len != write(logfd, logbuf, len)) {
		if (DEBUG_LEVEL(0)) {
			fprintf(stderr, "Can't write to log, terminating!\n");
			perror(Logfile);
		}
		exit(39);		/* Terminate if we can't log */
	}

#ifdef LOGCLOSE
	if (close(logfd) != 0) {
		if (DEBUG_LEVEL(0)) {
			fprintf(stderr, "Can't close log, terminating!\n");
			perror(Logfile);
		}
		exit(39);		/* Terminate if we can't log */
	}
	logfd = -1;
#endif
}



/*
 * getseqnum() returns a 4-character string which is a sequence number
 * that we should be able to use for creating files.  It is recommended
 * that such files be created with a mode that fails if the file exists.
 * If this happens, come back and get another sequence number.  This
 * will fail "soft" if we happen to get a duplicated sequence number.
 *
 * If getseqnum() detects any uncorrectable problems, it exit()s.
 */
char *
getseqnum()
{
	static char seqnum[WSUFSIZE       - 1  +1     + SLOP];
	/*                 file suffix, Grade, terminating null */
	register int n, i;
	FILE *fs;
	int tries = 20;

	/* Try up to 20 times to get the lock for the sequence file. */

	while (uucplock(SEQ_LOCK) != SUCCESS) {
		if (--tries <= 0) {
			DEBUG(0, "Can't lock seq file\n", "" );
			exit_gnuucp(0, 1);
		}
		DEBUG(3, "Sleeping for lock file\n", "");
		sleep(1);			/* Wait a second */
	}

	/* We have it locked -- now mess with it */

	if((fs = fopen(Seqfile, "r+")) == NULL) {
		perror(Seqfile);
		if((fs = fopen(Seqfile, "w")) == NULL) {
			DEBUG(0, "Could not create seq file %s\n", Seqfile);
			uucpunlock();
			exit_gnuucp(0, 1);
		} else {
			DEBUG(3, "Created new sequence file %s.\n", Seqfile);
			n = 3840;		/* Nice number to start at */
		}
	} else {
		if(fread((char *)seqnum,sizeof(char),sizeof(seqnum),fs)
		   != WSUFSIZE-1) {
corrupt:
			seqnum[sizeof(seqnum)-1] = '\0';  /* Null-term */
			DEBUG(0, "Corrupted sequence file: %s\n", seqnum);
			uucpunlock();
			exit_gnuucp(0, 1);
		} else {
			/*
			 * Parse out the sequence number, with heavy error
			 * checking
			 */
			n = 0;
			for (i = 0; i < WSUFSIZE-1; i++) {
				if(seqnum[i] >= '0' && seqnum[i] <= '9')
					n = 16 * n + seqnum[i] - '0';
				else if(seqnum[i] >= 'A' && seqnum[i] <= 'F')
					n = 16 * n + seqnum[i] - 'A' + 10;
				else if(seqnum[i] >= 'a' && seqnum[i] <= 'f')
					n = 16 * n + seqnum[i] - 'a' + 10;
				else goto corrupt;
			}
			n = (n + 1) &0xFFFF;	/* Increment to use */
		}
		rewind(fs);
	}
	sprintf(seqnum,"%04x",n);
	n = strlen(seqnum);
	if (fwrite((char *)seqnum, sizeof(char), n, fs) != n ||
	    EOF == fclose(fs)) {
		DEBUG(0, "Can't rewrite sequence file %s\n", Seqfile);
		uucpunlock();
		exit_gnuucp(0, 1);
	}

	uucpunlock();		/* Unlock sequence file */
	DEBUG(4, "Allocated sequence number %s\n", seqnum);

	log_sequence = seqnum;	/* Record for logging */
	return seqnum;
}
