/* $Header: uuxqt.c,v 1.4 90/02/22 19:43:43 gnu Exp $ */
/*  uuxqt.c: Lol Grant. 23rd. Sept 1987.
	     read uucp C.<hostname>XXXXX files and execute the
	     commands in them.
*/

#include "includes.h"
#include "uucp.h"
#include "sysdep.h"

extern s_or_f read_params();
extern void logit();

#ifdef VMS
#define EXEDIR  "user2:[uucp.uuslave]"	/* uuxqt executables live here */
#define NULL_DEVICE "NL:"
#define fork vfork
#define STATUS int
#define postmaster "system"
#define EXIT_OK 1
#define EXIT_ERR 0
#else
#define EXEDIR  "/usr/vaporwere/usr2/lol/src/uuslave"	/* uuxqt executables
							 * live here */
#define NULL_DEVICE "/dev/null"
#include <errno.h>
#include <sys/wait.h>
#define STATUS union wait
#define postmaster "postmaster"
#define EXIT_OK 0
#define EXIT_ERR 1
#endif

main(argc, argv)
int argc;
char **argv;
{
    int i;
    char command[60], addressee[60], input[60], output[60], line[132];
    char file[NAMESIZE];
    char *xfile;
    char *work_next();
    FILE *fd;

    for (i = 1; i < argc; i++) {
	if (argv[i][0] != '-')
	    break;
	switch (argv[i][1]) {

	default:
	    printf("uuxqt: unknown flag ignored\n");

	case 'x':
	    debug = atoi(&argv[i][2]);
	    printf("uuxqt: debug level set to %d\n", debug);
	    break;
	}
    }

    cuserid(who);		/* Finger out who we are */
    strcpy(host_name, "uuxqt");	/* We know who this is */

    if (read_params((char *)0) != SUCCESS)
	exit(EXIT_ERR);

    if (chdir(Spool) != 0) {
	fprintf(stderr, "can't change to %s\n", Spool);
	logit("FAILED", "can't change to Spool");
	exit(EXIT_ERR);
    }

    if (!work_scan((char *)NULL, "X")) {
	logit("Completed", "No Work");
	exit(EXIT_OK);
    }
    logit("OK", "startup");
    while ((xfile = work_next()) != NULL) {
	strcpy(file, munge_filename(xfile));
	if ((fd = fopen(file, "r")) == NULL) {
	    logit("CAN'T READ", file);
	    if (remove(file) != 0)
		logit("CAN'T REMOVE, EITHER", file);
	    continue;
	}
	input[0] = output[0] = 0;

	while (fgets(line, sizeof(line), fd) != NULL) {
	    line[strlen(line) - 1] = '\0';
	    DEBUG(1, "uuxqt:  %s\n", line);
	    switch (line[0]) {
	    case 'U':
		break;
	    case 'I':
		sprintf(input, "%s", &line[2]);
		DEBUG(1, "uuxqt: input %s\n", input);
		break;
	    case 'O':
		sprintf(output, "%s", &line[2]);
		DEBUG(1, "uuxqt: output %s\n", output);
		break;
	    case 'C':
		sscanf(&line[2], "%s %s", command, addressee);
		DEBUG(1, "uuxqt: command %s\n", command);
		DEBUG(1, "uuxqt: addressee %s\n", addressee);
		break;
	    case 'R':
		break;
	    default:
		break;
	    }
	}
	fclose(fd);

	if (invoke(command, addressee, input, output)) {
	    sprintf(line, "%s %s %s %s", command, addressee, input, output);
	    logit("FAILED", line);
	    if (invoke(command, postmaster, input, output)) {
		sprintf(line, "%s %s %s %s", command, postmaster, input, output);
		logit("FAILED", line);
		continue;
	    }
	}
	if (remove(file) == 0)
	    DEBUG(1, "deleted %s\n", file);
	else
	    DEBUG(1, "Can't delete %s\n", file);

	if (strcmp(input, NULL_DEVICE))
	    if (remove(input) == 0)
		DEBUG(1, "deleted %s\n", input);
	    else
		DEBUG(1, "Can't delete %s\n", input);

	if (strcmp(output, NULL_DEVICE))
	    if (remove(output) == 0)
		DEBUG(1, "deleted %s\n", output);
	    else
		DEBUG(1, "Can't delete %s\n", output);
    }
    work_done();

    logit("OK", "finished");
    exit(EXIT_OK);
}


/* Invoke the local program. Return its status (0 == ok) or -1 */
invoke(command, addressee, input, output)
char *command, *addressee, *input, *output;
{
    int i, pid;
    char tmp[255];
    STATUS status;

    if ((pid = fork())) {
	DEBUG(1, "fork = %d", pid);
	while ((i = wait(&status)) != pid)	/* null */
	    ;
	if (i == -1)
	    return 1;
#ifdef VMS
	DEBUG(1, "uuxqt status=%x\n", status);
	return (status & 1 != 1);
#else
	DEBUG(1, "uuxqt status=%x\n", (status.w_status >> 8) & 0377);
	return ((status.w_status >> 8) & 0377);
#endif
    }
#ifdef VMS
    sprintf(tmp, "%s%s.exe", EXEDIR, command);
#else
    sprintf(tmp, "%s/%s", EXEDIR, command);
#endif
    DEBUG(1, "execl %s\n", tmp);

    (void) close(0);
    /* FIXME, need to munge these filenames */
    if (open(input[0]? input: NULL_DEVICE, O_RDONLY) != 0)
	DEBUG(1, "cannot open %s as stdin\n", input);

    (void) close(1);
    if (open(input[0]? input: NULL_DEVICE, O_WRONLY) != 1)
	DEBUG(1, "cannot open %s as stdout\n", output);

    execl(tmp, command, addressee, 0);
    return -1;
}
